@@ -18,7 +18,6 @@ if djangoVersion[:2] >= (1, 4):
     from django.utils.timezone import is_aware, is_naive, utc, make_naive, get_default_timezone
     import datetime
 
-from django.db import utils
 from django.conf import settings
 from django.db.backends import *
 from django.db.backends.signals import connection_created
@@ -29,6 +28,10 @@ from sqlany_django.validation import DatabaseValidation
 
 from django.utils.safestring import SafeString, SafeUnicode
 
+
+DatabaseError = Database.DatabaseError
+IntegrityError = Database.IntegrityError
+
 Database.register_converter(Database.DT_TIMESTAMP, util.typecast_timestamp)
 Database.register_converter(Database.DT_DATE, util.typecast_date)
 Database.register_converter(Database.DT_TIME, util.typecast_time)
@@ -80,47 +83,37 @@ class CursorWrapper(object):
         if djangoVersion[:2] >= (1, 4) and settings.USE_TZ:
             args = _datetimes_in(args)
         try:
-            try:
-                if args != None:
-                    query = self.convert_query(query, len(args))
-                ret = self.cursor.execute(trace(query), trace(args))
-                return ret
-            except Database.OperationalError, e:
-                # Map some error codes to IntegrityError, since they seem to be
-                # misclassified and Django would prefer the more logical place.
-                if e[0] in self.codes_for_integrityerror:
-                    raise Database.IntegrityError(tuple(e))
-                raise
-        except Database.IntegrityError, e:
-            raise utils.IntegrityError(e)
-        except Database.DatabaseError, e:
-            raise utils.DatabaseError(e)
+            if args != None:
+                query = self.convert_query(query, len(args))
+            ret = self.cursor.execute(trace(query), trace(args))
+            return ret
+        except Database.OperationalError, e:
+            # Map some error codes to IntegrityError, since they seem to be
+            # misclassified and Django would prefer the more logical place.
+            if e[0] in self.codes_for_integrityerror:
+                raise Database.IntegrityError(tuple(e))
+            raise
 
     def executemany(self, query, args):
         if djangoVersion[:2] >= (1, 4) and settings.USE_TZ:
             args = tuple(_datetimes_in(arg) for arg in args)
         try:
             try:
-                try:
-                    len(args)
-                except TypeError:
-                    args = tuple(args)
-                if len(args) > 0:
-                    query = self.convert_query(query, len(args[0]))
-                    ret = self.cursor.executemany(trace(query), trace(args))
-                    return trace(ret)
-                else:
-                    return None
-            except Database.OperationalError, e:
-                # Map some error codes to IntegrityError, since they seem to be
-                # misclassified and Django would prefer the more logical place.
-                if e[0] in self.codes_for_integrityerror:
-                    raise Database.IntegrityError(tuple(e))
-                raise
-        except Database.IntegrityError, e:
-            raise utils.IntegrityError(e)
-        except Database.DatabaseError, e:
-            raise utils.DatabaseError(e)
+                len(args)
+            except TypeError:
+                args = tuple(args)
+            if len(args) > 0:
+                query = self.convert_query(query, len(args[0]))
+                ret = self.cursor.executemany(trace(query), trace(args))
+                return trace(ret)
+            else:
+                return None
+        except Database.OperationalError, e:
+            # Map some error codes to IntegrityError, since they seem to be
+            # misclassified and Django would prefer the more logical place.
+            if e[0] in self.codes_for_integrityerror:
+                raise Database.IntegrityError(tuple(e))
+            raise
 
     def fetchone(self):
         if djangoVersion[:2] < (1, 4) or not settings.USE_TZ:
@@ -168,7 +161,7 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     supports_regex_backreferencing = False
     supports_sequence_reset = False
     update_can_self_select = False
-    uses_custom_query_class = True
+    uses_custom_query_class = False
 
 class DatabaseOperations(BaseDatabaseOperations):
     compiler_module = "sqlany_django.compiler"
@@ -213,6 +206,47 @@ class DatabaseOperations(BaseDatabaseOperations):
             sql = "CAST(DATEFORMAT(%s, '%s') AS DATETIME)" % (field_name, format_str)
         return sql
 
+    def datetime_extract_sql(self, lookup_type, field_name, tzname):
+        """
+        Given a lookup_type of 'year', 'month', 'day', 'hour', 'minute' or
+        'second', returns the SQL that extracts a value from the given
+        datetime field field_name, and a tuple of parameters.
+        """
+        if settings.USE_TZ:
+            params = [tzname]
+        else:
+            params = []
+        if lookup_type == 'week_day':
+            # Returns an integer, 1-7, Sunday=1
+            sql = "DATEFORMAT(%s, 'd')" % field_name
+        else:
+            # YEAR(), MONTH(), DAY(), HOUR(), MINUTE(), SECOND() functions
+            sql = "%s(%s)" % (lookup_type.upper(), field_name)
+        return sql,params
+
+    def datetime_trunc_sql(self, lookup_type, field_name, tzname):
+        """
+        Given a lookup_type of 'year', 'month', 'day', 'hour', 'minute' or
+        'second', returns the SQL that truncates the given datetime field
+        field_name to a datetime object with only the given specificity, and
+        a tuple of parameters.
+        """
+        fields = ['year', 'month', 'day', 'hour', 'minute', 'second']
+        format = ('YYYY-', 'MM', '-DD', 'HH:', 'NN', ':SS') # Use double percents to escape.
+        format_def = ('0000-', '01', '-01', ' 00:', '00', ':00')
+        if settings.USE_TZ:
+            params = [tzname]
+        else:
+            params = []
+        try:
+            i = fields.index(lookup_type) + 1
+        except ValueError:
+            sql = field_name
+        else:
+            format_str = ''.join([f for f in format[:i]] + [f for f in format_def[i:]])
+            sql = "CAST(DATEFORMAT(%s, '%s') AS DATETIME)" % (field_name, format_str)
+        return sql,params
+
     def deferrable_sql(self):
         return " CHECK ON COMMIT"
 
@@ -333,7 +367,7 @@ class DatabaseOperations(BaseDatabaseOperations):
             for sequence in sequences:
                 sql.append('call sa_reset_identity(\'%s\', NULL, 0);' % sequence['table'])
             
-            sql.append('SET TEMPORARY OPTION wait_for_commit = \'Off\'')
+            sql.append('SET TEMPORARY OPTION wait_for_commit = \'Off\';')
             sql.append('COMMIT;')
             
             return sql
@@ -388,6 +422,8 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         'iendswith': "LIKE %s ESCAPE '\\'"
     }
 
+    Database = Database
+    
     def __init__(self, *args, **kwargs):
         super(DatabaseWrapper, self).__init__(*args, **kwargs)
 
@@ -403,7 +439,10 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         self.client = DatabaseClient(self)
         self.creation = DatabaseCreation(self)
         self.introspection = DatabaseIntrospection(self)
-        self.validation = DatabaseValidation(self)
+        if djangoVersion[:2] >= (1, 2):
+            self.validation = DatabaseValidation(self)
+        else:
+            self.validation = DatabaseValidation()
 
     def _valid_connection(self):
         if self.connection is not None:
@@ -419,54 +458,87 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         self.cursor().execute('PREPARE TO COMMIT')
 
     def _cursor(self):
+        return self.create_cursor()
+
+    def _rollback(self):
+        try:
+            BaseDatabaseWrapper._rollback(self)
+        except Database.NotSupportedError:
+            pass
+
+    # New methods for Django 1.6
+    def get_connection_params(self):
+        kwargs = {}
+        links = {}
+
+        settings_dict = self.settings_dict
+
+        def setting( key ):
+            if settings_dict.has_key(key):
+                return settings_dict[key]
+            if settings_dict.has_key('DATABASE_%s' % key):
+                return settings_dict['DATABASE_%s' % key]
+            return None
+        #
+
+        uid = setting( 'USER' )
+        if uid is not None:
+            kwargs['uid'] = uid
+        dbn = setting( 'NAME' )
+        if dbn is not None:
+            kwargs['dbn'] = dbn
+        pwd = setting( 'PASSWORD' )
+        if pwd is not None:
+            kwargs['pwd'] = pwd
+
+        root = Database.Root('PYTHON')
+
+        try:
+            vers = root.api.sqlany_client_version()
+            ret = True
+        except:
+            length = 1000
+            buffer = ctypes.create_string_buffer(length)
+            ret = root.api.sqlany_client_version(ctypes.byref(buffer), length)
+            vers = buffer.value
+        if ret:
+            vers = int(vers.split('.')[0])
+        else:
+            vers = 11 # assume old
+        host = setting( 'HOST' )
+        if host == '':
+            host = 'localhost' # "Set to empty string for localhost"
+        if host is not None and vers > 11:
+            kwargs['host'] = host
+            port = setting( 'PORT' )
+            if port is not None:
+                kwargs['host'] += ':%s' % port
+        else:
+            if host is not None:
+                links['host'] = host
+            port = setting( 'PORT' )
+            if port is not None:
+                links['port'] = str( port )
+        if len(links) > 0:
+            kwargs['links'] = 'tcpip(' + ','.join(k+'='+v for k, v in links.items()) + ')'
+        kwargs.update(setting( 'OPTIONS' ))
+        return kwargs
+
+    def get_new_connection( self, conn_params ):
+        return Database.connect(**conn_params)
+        
+    def init_connection_state( self ):
+        if self.settings_dict['AUTOCOMMIT']:
+            self.set_autocommit(self.settings_dict['AUTOCOMMIT'])
+
+    def create_cursor( self ):
         cursor = None
         if not self._valid_connection():
-            kwargs = {}
-            links = {}
-            settings_dict = self.settings_dict
-            if settings_dict['USER']:
-                kwargs['uid'] = settings_dict['USER']
-            if settings_dict['NAME']:
-                kwargs['dbn'] = settings_dict['NAME']
-            if settings_dict['PASSWORD']:
-                kwargs['pwd'] = settings_dict['PASSWORD']
-
-            root = Database.Root('PYTHON')
-
-            try:
-                vers = root.api.sqlany_client_version()
-                ret = True
-            except:
-                length = 1000
-                buffer = ctypes.create_string_buffer(length)
-                ret = root.api.sqlany_client_version(ctypes.byref(buffer), length)
-                vers = buffer.value
-            if ret:
-                vers = int(vers.split('.')[0])
-            else:
-                vers = 11 # assume old
-            host = settings_dict['HOST']
-            if host == '':
-                host = 'localhost' # "Set to empty string for localhost"
-            if host and vers > 11:
-                kwargs['host'] = host
-                try:
-                    port = str(settings_dict['PORT'])
-                except:
-                    port = None
-                if port:
-                    kwargs['host'] += ':%s' % port
-            else:
-                if host:
-                    links['host'] = host
-                if settings_dict['PORT']:
-                    links['port'] = str(settings_dict['PORT'])
-            if len(links) > 0:
-                kwargs['links'] = 'tcpip(' + ','.join(k+'='+v for k, v in links.items()) + ')'
-            kwargs.update(settings_dict['OPTIONS'])
-            self.connection = Database.connect(**kwargs)
+            kwargs = self.get_connection_params()
+            self.connection = self.get_new_connection(kwargs)
             cursor = CursorWrapper(self.connection.cursor())
-            cursor.execute("SET OPTION PUBLIC.reserved_keywords='LIMIT'")
+            if djangoVersion[:2] < (1, 2):
+                cursor.execute("SET TEMPORARY OPTION PUBLIC.reserved_keywords='LIMIT'")
             cursor.execute("SET TEMPORARY OPTION TIMESTAMP_FORMAT='YYYY-MM-DD HH:NN:SS.SSSSSS'")
             connection_created.send(sender=self.__class__, connection=self)
         if not cursor:
@@ -474,8 +546,19 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
         return cursor
 
-    def _rollback(self):
-        try:
-            BaseDatabaseWrapper._rollback(self)
-        except Database.NotSupportedError:
-            pass
+    def _set_autocommit( self, autocommit ):
+        """
+        Backend-specific implementation to enable or disable autocommit.
+        """
+        curs = self.create_cursor()
+        curs.execute( "SET TEMPORARY OPTION chained='%s'" %
+                      ('Off' if autocommit else 'On') )
+        curs.close()
+
+    def is_usable(self):
+        """
+        Tests if the database connection is usable.
+        This function may assume that self.connection is not None.
+        """
+        return self._valid_connection()
+#
